;**********************************************************************
;                                                                     *
;    Filename:         MDLab FC Main.asm                              *
;    Date:             6/10/12                                        *
;    File Version:     .01 Beta                                       *
;                                                                     *
;    Author:           David Hoffman                                  *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: P16F882.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;    code listed on this page is licensed under the Creative Commons  *
;    Attribution-NonCommercial 3.0 Unported License                   *
;                                                                     *
;    For commercial use of this code, please contact David Hoffman    *
;    at dhoffman@neoloch.com .                                        *
;**********************************************************************


     LIST      p=16F1516            ; list directive to define processor
     #INCLUDE <p16F1516.inc>        ; processor specific variable definitions

	__CONFIG _CONFIG1, _FCMEN_OFF & _IESO_OFF & _BOREN_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDTE_OFF & _FOSC_INTOSC
	__CONFIG _CONFIG2, _WRT_OFF & _BORV_HI & _LVP_OFF

	ERRORLEVEL -302

#DEFINE		LED_PORT	PORTC
#DEFINE		LED_SEG_A	PORTC,1
#DEFINE		LED_SEG_B	PORTC,2
#DEFINE		LED_SEG_C	PORTC,3
#DEFINE		LED_SEG_D	PORTC,4
#DEFINE		LED_SEG_E	PORTC,5
#DEFINE		LED_SEG_F	PORTC,6
#DEFINE		LED_SEG_G	PORTC,7
#DEFINE		LED_SEG_DOT	PORTA,5
#DEFINE		LED_DIG1	PORTB,0
#DEFINE		LED_DIG2	PORTB,1
#DEFINE		LED_DIG3	PORTB,2
#DEFINE		LED_DIG4	PORTB,3
#DEFINE		LED_DIG5	PORTB,4
#DEFINE		LED_L1		PORTC,1
#DEFINE		LED_L2		PORTC,2
#DEFINE		LED_L3		PORTC,3
#DEFINE		FC_PERIOD	D'209'	;DEFAULT COUNT PERIOD.



; UDATA declares a section of uninitialized data
SYSTEM_VAR	UDATA	; VARIABLES is the name of the section of memory
KEY_PRESSED	RES 1	; uninitialized data, placed by linker in GPR's.

SYSTEM_VAR_SHR	UDATA_SHR
DEBUG		RES 1	;RESERVED FOR DEBUG MODE.
DISP_SHOW_DIG	RES 1
DISP_TEMP	RES 1
DISP_COUNTER_L	RES 1
DISP_COUNTER_M	RES 1
DISP_COUNTER_H	RES 1
DISP_TIMER	RES 1
DISP_MEMORY_1	RES 1
DISP_MEMORY_2	RES 1
DISP_MEMORY_3	RES 1
DISP_MEMORY_4	RES 1
DISP_LOOP	RES 1
DISP_CALC_1	RES 1
DISP_CALC_2	RES 1
DISP_CALC_3	RES 1
DISP_DOTS	RES 1
; BIT 7 = L1
; BIT 6 = L2
; BIT 5 = L3
; BIT 4 = DIGIT 1 DOT
; BIT 3 = DIGIT 2 DOT
; BIT 2 = DIGIT 3 DOT
; BIT 1 = DIGIT 1 DOT


;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RESET_VECTOR    CODE   0x0000
          goto    START             ; When using debug header, ICD2 may not stop
                                    ; on instruction 0 during reset.
;------------------------------------------------------------------------------
; SYSTEM CONFIGURE
;------------------------------------------------------------------------------
START	ORG 0X0008
	CALL	SYSTEM_CONFIG
	CLRF	DISP_COUNTER_L
	CLRF	DISP_COUNTER_M
	CLRF	DISP_COUNTER_H
	CLRF	DISP_SHOW_DIG
	CLRF	DISP_TIMER
	CLRF	DISP_DOTS
	CALL	DISP_CLEAR_COUNTER
	CLRF	DISP_MEMORY_1
	CLRF	DISP_MEMORY_2
	CLRF	DISP_MEMORY_3
	CLRF	DISP_MEMORY_4

;------------------------------------------------------------------------------
; MAIN ROUTINE
;------------------------------------------------------------------------------
MAIN
	CALL	DISP_READ_COUNT
	CALL	DISP_SHOW_COUNT
	CALL	DISP_TMR1_OVERFLOW
	GOTO	MAIN
;------------------------------------------------------------------------------
; CHECK FOR TMR1 OVERFLOW - THIS ROUTINE MONITORS THE OVERFLOW INTERRUPT FLAG
; FOR TIMER1 AND IF DETECTED INCREMENTS A SECOND COUNTER AND THEN RESETS
; THE INTERRUPT FLAG.
;------------------------------------------------------------------------------
DISP_TMR1_OVERFLOW
	BANKSEL	PIR1
	BTFSS	PIR1,TMR1IF
	RETURN
	INCF	DISP_COUNTER_H,F
	BCF	PIR1,TMR1IF
	RETURN

;------------------------------------------------------------------------------
; READ COUNT DATA - THIS ROUTINE CHECKS TIMER 2 TO SEE IF THE COUNTING PERIOD
; HAS EXPIRED AND IF IT HAS IT THEN COPIES THE CONTENTS OF TIMER 1 INTO
; DISP_COUNTER_L AND DISP_COUNTER_M
;------------------------------------------------------------------------------
DISP_READ_COUNT
	BANKSEL	PIR1
	BTFSS	PIR1,TMR2IF		;IS TIMER INTERRUPT SET?
	RETURN				;NO
	BCF	PIR1,TMR2IF		;YES - CLEAR INTERRUPT BIT.

	INCF	DISP_TIMER,F		;INCREMENT DISP_TIMER
	MOVLW	D'100'
	SUBWF	DISP_TIMER,W		;SUBTRACT 100 FROM DISP_TIMER
	BTFSS	STATUS,Z		;LOOP = 100?
	RETURN				;NO

	CLRF	DISP_TIMER
	BANKSEL	TMR1L
	MOVF	TMR1L,W			;MOVE TMR1L TO W.
	MOVWF	DISP_COUNTER_L		;MOVE W TO DISP_COUNTER_L
	MOVF	TMR1H,W			;MOVE TMR1H TO W.
	MOVWF	DISP_COUNTER_M		;MOVE W TO DISP_COUNTER_M

	CALL	DISP_CONVERT		;CONVERT BINARY VALUES TO BCD
	CALL	DISP_CLEAR_COUNTER
	RETURN				;NO - COUNT IS LESS THAN

;------------------------------------------------------------------------------
; UPDATE DISPLAY - DEPENDING ON THE VALUE OF DISP_SHOW_DIG, THE APPROPRIATE
; DIGIT ON THE DISPLAY IS UPDATED.
;------------------------------------------------------------------------------
DISP_SHOW_COUNT
	MOVLW	0X00
	SUBWF	DISP_SHOW_DIG,W		;UPDATE DIGIT 1?
	BTFSC	STATUS,Z
	GOTO	DISPLAY_COUNT_D1	;YES

	MOVLW	0X01			;UPDATE DIGIT 2?
	SUBWF	DISP_SHOW_DIG,W
	BTFSC	STATUS,Z
	GOTO	DISPLAY_COUNT_D2	;YES

	MOVLW	0X02
	SUBWF	DISP_SHOW_DIG,W
	BTFSC	STATUS,Z
	GOTO	DISPLAY_COUNT_D3

	MOVLW	0X03
	SUBWF	DISP_SHOW_DIG,W
	BTFSC	STATUS,Z
	GOTO	DISPLAY_COUNT_D4
	CLRF	DISP_SHOW_DIG
	GOTO	DISPLAY_COUNT_D5

;--------------------------------------------------------------------------
; UPDATE DIGIT 1
;--------------------------------------------------------------------------
DISPLAY_COUNT_D1
	MOVF	DISP_MEMORY_1,W		;COPY DISP_MEMORY_1 TO W.
	CALL	DISPLAY_HEX_7S		;GET 7 SEGMENT CODE TO DISPLAY.
	BANKSEL	LED_PORT
	BCF	LED_DIG5		;TURN OFF DIGIT 5 (DOTS).
	CALL	DISPLAY_CHARACTER	;PUT NEW DATA ON DISPLAY BITS.

	BSF	LED_SEG_DOT		;TURN OFF DOT.
	BTFSC	DISP_DOTS,4		;IS DOT FLAG SET?
	BCF	LED_SEG_DOT		;YES - TURN ON DOT.

	BANKSEL	LED_PORT
	BSF	LED_DIG1		;TURN ON DIGIT 1.
	INCF	DISP_SHOW_DIG,F		;INC SHOW_DIG FOR NEXT DIGIT.
	RETURN
;--------------------------------------------------------------------------
; UPDATE DIGIT 2
;--------------------------------------------------------------------------
DISPLAY_COUNT_D2
	MOVF	DISP_MEMORY_2,W
	CALL	DISPLAY_HEX_7S
	BANKSEL	LED_PORT
	BCF	LED_DIG1
	CALL	DISPLAY_CHARACTER

	BSF	LED_SEG_DOT
	BTFSC	DISP_DOTS,3
	BCF	LED_SEG_DOT

	BANKSEL	LED_PORT
	BSF	LED_DIG2
	INCF	DISP_SHOW_DIG,F
	RETURN
;--------------------------------------------------------------------------
; UPDATE DIGIT 3
;--------------------------------------------------------------------------
DISPLAY_COUNT_D3
	MOVF	DISP_MEMORY_3,W
	CALL	DISPLAY_HEX_7S
	BANKSEL	LED_PORT
	BCF	LED_DIG2
	CALL	DISPLAY_CHARACTER

	BSF	LED_SEG_DOT
	BTFSC	DISP_DOTS,2
	BCF	LED_SEG_DOT

	BANKSEL	LED_PORT
	BSF	LED_DIG3
	INCF	DISP_SHOW_DIG,F
	RETURN
;--------------------------------------------------------------------------
; UPDATE DIGIT 4
;--------------------------------------------------------------------------
DISPLAY_COUNT_D4
	MOVF	DISP_MEMORY_4,W
	CALL	DISPLAY_HEX_7S
	BANKSEL	LED_PORT
	BCF	LED_DIG3
	CALL	DISPLAY_CHARACTER

	BSF	LED_SEG_DOT
	BTFSC	DISP_DOTS,1
	BCF	LED_SEG_DOT

	BANKSEL	LED_PORT
	BSF	LED_DIG4
	INCF	DISP_SHOW_DIG,F
	RETURN
;--------------------------------------------------------------------------
; UPDATE DIGIT 5 (ADDITIONAL DOTS ON DISPLAY)
;--------------------------------------------------------------------------
DISPLAY_COUNT_D5
	BANKSEL	LED_PORT
	BCF	LED_DIG4
	BSF	LED_L1
	BSF	LED_L2
	BSF	LED_L3
	BTFSC	DISP_DOTS,7
	BCF	LED_L1
	BTFSC	DISP_DOTS,6
	BCF	LED_L2
	BTFSC	DISP_DOTS,5
	BCF	LED_L3
	BANKSEL	LED_PORT
	BSF	LED_DIG5
	RETURN
;--------------------------------------------------------------------------
; DISPLAY CURRENT CHARACTER DATA ON OUPUT PORT. THIS METHOD WAS CHOSEN
; OVER A SIMPLE MOVE SO THAT OTHER PORT / BIT COMBINATIONS COULD EASILY
; BE IMPLEMENTED SHOULD OTHER RESOURCES ON THE MCU NEED TO BE FREED UP.
;--------------------------------------------------------------------------
DISPLAY_CHARACTER
	MOVWF	DISP_TEMP
	BSF	LED_SEG_A
	BTFSS	DISP_TEMP,0
	BCF	LED_SEG_A

	BSF	LED_SEG_B
	BTFSS	DISP_TEMP,1
	BCF	LED_SEG_B

	BSF	LED_SEG_C
	BTFSS	DISP_TEMP,2
	BCF	LED_SEG_C

	BSF	LED_SEG_D
	BTFSS	DISP_TEMP,3
	BCF	LED_SEG_D

	BSF	LED_SEG_E
	BTFSS	DISP_TEMP,4
	BCF	LED_SEG_E

	BSF	LED_SEG_F
	BTFSS	DISP_TEMP,5
	BCF	LED_SEG_F

	BSF	LED_SEG_G
	BTFSS	DISP_TEMP,6
	BCF	LED_SEG_G

	RETURN
;--------------------------------------------------------------------------
; ASCII LOOKUP TABLE. COMMON ANODE - RETURN A 7 BIT CODE FOR THE ACTUAL
; DISPLAYALBE CHARACTERS ON THE SEVEN SEGMENT DISPLAY.
;--------------------------------------------------------------------------
DISPLAY_HEX_7S
	MOVWF	DISP_TEMP
	MOVLW	B'00001111'
	ANDWF	DISP_TEMP,W
	ADDWF	PCL,F
	RETLW 	0XC0
	RETLW 	0XF9
	RETLW 	0XA4
	RETLW 	0XB0
	RETLW 	0X99
	RETLW 	0X92
	RETLW 	0X82
	RETLW 	0XF8
	RETLW 	0X80
	RETLW 	0X98
	RETLW 	0X88
	RETLW 	0X83
	RETLW 	0XC6
	RETLW 	0XA1
	RETLW 	0X86
	RETLW 	0X8E

;==========================================================================
;--------------------------------------------------------------------------
; CONVERT COUNT FROM BINARY TO BCD.
;--------------------------------------------------------------------------
DISP_CONVERT
	CLRF	DISP_CALC_1
	CLRF	DISP_CALC_2
	CLRF	DISP_CALC_3
	MOVLW	D'24'
	MOVWF	DISP_LOOP

DISP_CONVERT_LOOP
	BCF	STATUS,C
	RLF	DISP_COUNTER_L,F
	RLF	DISP_COUNTER_M,F
	RLF	DISP_COUNTER_H,F
	RLF	DISP_CALC_3,F
	RLF	DISP_CALC_2,F
	RLF	DISP_CALC_1,F
	DECFSZ	DISP_LOOP,F
	GOTO	DISP_CONVERT_DEC
	GOTO	DISP_CONVERT_S2

DISP_CONVERT_DEC
	MOVLW	DISP_CALC_3
	MOVWF	FSR0
	CALL	DISP_CONVERT_BCD
	MOVLW	DISP_CALC_2
	MOVWF	FSR0
	CALL	DISP_CONVERT_BCD
	MOVLW	DISP_CALC_1
	MOVWF	FSR0
	CALL	DISP_CONVERT_BCD
	GOTO	DISP_CONVERT_LOOP

DISP_CONVERT_BCD
	MOVLW	0X03
	ADDWF	INDF0,W
	MOVWF	DISP_TEMP
	BTFSC	DISP_TEMP,3
	MOVWF	INDF0
	MOVLW	0X30
	ADDWF	INDF0,W
	MOVWF	DISP_TEMP
	BTFSC	DISP_TEMP,7
	MOVWF	INDF0
	RETURN

;--------------------------------------------------------------------------
; MOVE BCD VALUES INTO CORRECT DISPLAY MEMORY LOCATIONS.
;--------------------------------------------------------------------------
DISP_CONVERT_S2
	MOVLW	0X0F
	ANDWF	DISP_CALC_3,W
	MOVWF	DISP_MEMORY_4

	MOVLW	0XF0
	ANDWF	DISP_CALC_3,W
	MOVWF	DISP_MEMORY_3
	SWAPF	DISP_MEMORY_3,F

	MOVLW	0X0F
	ANDWF	DISP_CALC_2,W
	MOVWF	DISP_MEMORY_2

	MOVLW	0XF0
	ANDWF	DISP_CALC_2,W
	MOVWF	DISP_MEMORY_1
	SWAPF	DISP_MEMORY_1,F
	CALL	DISP_CLEAR_DP
	BSF	DISP_DOTS,4

; DOUBLE DIGIT MOVE CHECK
	MOVLW	0XF0
	ANDWF	DISP_CALC_1,W
	BTFSC	STATUS,Z		;IS SIXTH DIGIT > ZERO?
	GOTO	DISP_CONVERT_S3		;NO
	CALL	DISP_DIGIT_SHIFT
	CALL	DISP_DIGIT_SHIFT

	MOVLW	0XF0
	ANDWF	DISP_CALC_1,W
	MOVWF	DISP_MEMORY_1
	SWAPF	DISP_MEMORY_1,F

	MOVLW	0X0F
	ANDWF	DISP_CALC_1,W
	MOVWF	DISP_MEMORY_2
	CALL	DISP_CLEAR_DP
	BSF	DISP_DOTS,2
	RETURN

DISP_CONVERT_S3			;SINGLE DIGIT MOVE
	MOVLW	0X0F
	ANDWF	DISP_CALC_1,W
	BTFSC	STATUS,Z
	RETURN			;DISP_CALC_3 = 0 SO DON'T CHANGE.
	CALL	DISP_DIGIT_SHIFT
	MOVLW	0X0F
	ANDWF	DISP_CALC_1,W
	MOVWF	DISP_MEMORY_1
	CALL	DISP_CLEAR_DP
	BSF	DISP_DOTS,3
	RETURN

DISP_CLEAR_DP
	BCF	DISP_DOTS,1
	BCF	DISP_DOTS,2
	BCF	DISP_DOTS,3
	BCF	DISP_DOTS,4
	RETURN
;------------------------------------------------------------------------------
; SHIFT DATA IN DISP_CSHOW_L AND CSHOW_H RIGHT ONE NIBBLE.
;------------------------------------------------------------------------------
DISP_DIGIT_SHIFT
	MOVF	DISP_MEMORY_3,W
	MOVWF	DISP_MEMORY_4
	MOVF	DISP_MEMORY_2,W
	MOVWF	DISP_MEMORY_3
	MOVF	DISP_MEMORY_1,W
	MOVWF	DISP_MEMORY_2
	RETURN


DISP_CLEAR_COUNTER
	BANKSEL	TMR1L
	CLRF	TMR1L
	CLRF	TMR1H
	CLRF	DISP_COUNTER_H
	RETURN

SYSTEM_CONFIG
;------------------------------------------------------------------------------
; SET CLOCK TO 16 MHz
;------------------------------------------------------------------------------
	BANKSEL	OSCCON
	MOVLW	0XF0
	IORWF	OSCCON,F
;------------------------------------------------------------------------------
; CONFIGURE PORTA
;------------------------------------------------------------------------------
	BANKSEL	PORTA
	CLRF	PORTA
	BANKSEL	LATA
	CLRF	LATA
	BANKSEL	ANSELA
	CLRF	ANSELA
	BANKSEL	TRISA
	MOVLW	0X10
	MOVWF	TRISA
;------------------------------------------------------------------------------
; CONFIGURE PORTB
;------------------------------------------------------------------------------
	BANKSEL	PORTB
	CLRF	PORTB
	BANKSEL	LATB
	CLRF	LATB
	BANKSEL	ANSELB
	CLRF	ANSELB
	BANKSEL	TRISB
	CLRF	TRISB
;------------------------------------------------------------------------------
; CONFIGURE PORTC
;------------------------------------------------------------------------------
	BANKSEL	PORTC
	CLRF	PORTC
	BANKSEL	LATC
	CLRF	LATC
	BANKSEL	ANSELC
	CLRF	ANSELC
	BANKSEL	TRISC
	MOVLW	B'00000001'
	MOVWF	PORTC
;------------------------------------------------------------------------------
; CONFIGURE TIMER1 (16 BIT TIMER) TO COUNT ON TIMER0 RESET.
;------------------------------------------------------------------------------
	BANKSEL	T1GCON
	MOVLW	B'01000000'
	MOVWF	T1GCON

	BANKSEL	T1CON
	MOVLW	B'10000101'
	MOVWF	T1CON
;------------------------------------------------------------------------------
; CONFIGURE TIMER2 (8 BIT TIMER) - THIS TIMER IS USED AS A CLOCK.
;------------------------------------------------------------------------------
	BANKSEL	T2CON
	MOVLW	B'00010111'
	MOVWF	T2CON

	BANKSEL	PR2
	MOVLW	FC_PERIOD
	MOVWF	PR2
	RETURN

	END